!{\src2tex{textfont=tt}}
!!****m* ABINIT/m_crystal
!! NAME
!! m_crystal
!!
!! FUNCTION
!! Module containing the definition of the crystal_structure data type and methods used to handle it. 
!!
!! COPYRIGHT
!!  Copyright (C) 2008-2010 ABINIT group (MG, YP)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! TODO 
!! It would be useful to have a set of
!! methods to dump the crystalline structure according to the file format 
!! used by an external graphical software ...
!!
!! PARENTS
!!
!! CHILDREN
!!  
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

MODULE m_crystal

 use defs_basis
 use m_errors 
#if defined HAVE_ETSF_IO
 use etsf_io
#endif

 use defs_datatypes,   only : Hdr_type
 use m_numeric_tools,  only : set2unit

 implicit none

 private 
!!***

!----------------------------------------------------------------------

!!****t* m_crystal/crystal_structure
!! NAME
!! crystal_structure
!! 
!! FUNCTION
!! Structure defining the unit cell (geometry, atomic positions and symmetry operations in real and reciprocal space)
!! 
!! SOURCE

 type,public :: crystal_structure

!scalars
  !integer :: point_group                    ! Point group
  !integer :: bravais,crystsys               ! Bravais lattice, Crystal system
  !integer :: nptsym                         ! No of point symmetries of the Bravais lattice
  !integer :: bravais(11)                    ! bravais(1)=iholohedry, bravais(2)=center
                                             ! bravais(3:11)=coordinates of rprim in the axes of the conventional
                                             ! bravais lattice (*2 if center/=0)
  !integer,pointer ptsymrel(:,:,:)
  !ptsymrel(3,3,nptsym)
  ! nptsym point-symmetry operations of the Bravais lattice in real space in terms of primitive translations.

  integer :: natom     
  ! Number of atoms

  integer :: nsym   
  ! Number of symmetry operations

  integer :: ntypat   
  ! Number of type of atoms

  !$integer :: ntypalch,ntyppure

  integer :: npsp  
  ! No. of pseudopotentials

  integer :: space_group 
  ! Space group

  integer :: timrev  
  ! 2 => take advantage of time-reversal symmetry. BE CAREFUL here, as the convention used in abinit is different.
  ! 1 => do not use time-reversal symmetry.

  real(dp) :: ucvol                          
  ! Real space unit cell volume.

  logical :: use_antiferro
  ! .TRUE. if AFM symmetries are present and used.

  logical :: has_inversion 
  ! .TRUE. if inversion symmetry is present

!arrays
  real(dp) :: angdeg(3)
  ! Angles among rprim (degree).

  real(dp) :: gmet(3,3)
  ! Reciprocal space metric ($\textrm{bohr}^{-2}$).

  real(dp) :: gprimd(3,3)
  ! Dimensional primitive translations for reciprocal space ($\textrm{bohr}^{-1}$)

  real(dp) :: rmet(3,3)
  ! Metric in real space.

  real(dp) :: rprimd(3,3)
  ! Direct lattice vectors, Bohr units.

  integer,pointer :: indsym(:,:,:)    SET2NULL
  ! indsym(4,nsym,natom)
  ! indirect indexing array for atoms, see symatm.F90.

  integer,pointer :: symafm(:)   SET2NULL
  ! symafm(nsym)
  ! (Anti)Ferromagnetic symmetries.

  integer,pointer :: symrec(:,:,:)   SET2NULL
  ! symrec(3,3,nsym)
  ! Symmetry operation in reciprocal space (reduced coordinates)

  integer,pointer :: symrel(:,:,:)   SET2NULL
  ! symrel(3,3,nsym)
  ! Symmetry operations in direct space (reduced coordinates).

  integer,pointer :: atindx(:)     SET2NULL
  integer,pointer :: atindx1(:)    SET2NULL
  ! atindx(natom), atindx1(natom)
  ! Index tables for atoms useful to treat atoms type after type.

  integer,pointer :: typat(:)    SET2NULL
  integer,pointer :: nattyp(:)   SET2NULL
  ! typat(natom), nattyp(ntypat)
  ! Type of each natom and number of atoms of each type.

  real(dp),pointer :: tnons(:,:)   SET2NULL
  ! tnons(3,nsym)
  ! Fractional translations (reduced coordinates)

  real(dp),pointer :: xcart(:,:)  SET2NULL

  real(dp),pointer :: xred(:,:)   SET2NULL
  ! xcart(3,natom), xred(3,natom)
  ! Cartesian and Reduced coordinates.

  real(dp),pointer :: spinrot(:,:)   SET2NULL
  ! spinrot(4,nsym)
  ! spinor rotation matrices.

! Useful quantities that might be added in the future
  !real(dp),pointer :: amu(:)                ! amu(ntypat)

  real(dp),pointer :: ziontypat(:)   SET2NULL
  ! ziontypat(ntypat)
  ! Charge of the pseudo-ion (No of valence electrons needed to screen exactly the pseudopotential).

  !real(dp),pointer :: znucltypat(:)         ! znucltypat(ntypat) from alchemy

  real(dp),pointer :: znucl(:)  SET2NULL
  ! znucl(npsp)
  ! Nuclear charge for each type of pseudopotential

  character(len=132),pointer :: title(:)   SET2NULL
   ! title(ntypat)
   ! The content of first line read from the psp file

 end type crystal_structure
!!***

!----------------------------------------------------------------------

 public :: init_crystal            ! Main Creation method.
 public :: init_crystal_from_hdr   ! Initialize the object from the abinit header.
 public :: nullify_crystal         ! Set all pointers to NULL.
 public :: destroy_crystal         ! Free the structure.
 public :: print_crystal           ! Print dimensions and basic info stored in the object
 public :: print_symmetries        ! Helper function to print symmetries in a nice format.
 public :: abi_crystal_put         ! Dump the object using netcdf. 
                                   ! TODO corresponding reading method is missing, at present use InitCrystalFromHdr
 public :: idx_spatial_inversion   ! Return the index of the spatial inversion, 0 if not present.
 public :: isymmorphic             ! Returns .TRUE. if space group is symmorphic.

 !integer,private,parameter :: SPATIAL_INVERSION = RESHAPE(/-1,0,0,0,-1,0,0,0,-1/),(/3.3/)

CONTAINS  !=========================================================================================================================
!!***

!!****f* m_crystal/init_crystal
!! NAME
!!  init_crystal 
!!
!! FUNCTION
!!  Initialize a crystal_structure data type.
!!  Ideally the routine should work in two different modes:
!!  Either the symmetries are directly supplied or the space group
!!  is determined starting from the definition of the unit cell.
!!  Only the first method is implemented, the second one should be
!!  a wrapper for the symmetry finder library. To implement the 
!!  second case I have to add additional entries in the object
!!  and I have also to pass an object describing the (optional) geometry builder.
!!
!! INPUTS
!!  natom=number of atom
!!  ntypat=number of type of atoms
!!  nsym=number of symmetry operations
!!  rprimd(3,3)=dimensional lattive vector (real space)
!!  typat(natom)=type of each atom
!!  xred(3,natom)=reduced coordinates of each atom
!!  symrel(3,3) [optional]=symmetry operations in real space
!!  space_group=Space group (0 if not available)
!!  tnons(3,nsym) [optional]=fractional Translations
!!  symafm(nsym) [optional]=  ferromagnetic symmetries
!!  remove_inv [optional]= if .TRUE. the inversion is removed from the set of symmetries
!!  timrev ==2 => take advantage of time-reversal symmetry
!!         ==1 ==> do not use time-reversal symmetry 
!!
!! OUTPUT
!!  Cryst<crystal_structure>= the object completely initialized.
!!
!! TODO
!!  Add additional entries in the class:
!!  1) Info on space and point group (generators?).
!!  2) alchemy
!!  3) masses and nuclear (pseudo&AE) charge
!!  4) forces stresses, velocities.
!!  5) constraints for the relaxation
!!  6) Likely I will need also info on the electric field and berryopt
!!
!! PARENTS
!!      m_crystal,vtorho
!!
!! CHILDREN
!!      atmdata,etsf_io_data_write,etsf_io_low_close,etsf_io_low_def_var
!!      etsf_io_low_error_to_str,etsf_io_low_open_modify
!!      etsf_io_low_set_write_mode,etsf_io_low_write_dim,etsf_io_low_write_var
!!      wrtout
!!
!! SOURCE

subroutine init_crystal(Cryst,space_group,natom,npsp,ntypat,nsym,rprimd,typat,xred,&
& ziontypat,znucl,timrev,use_antiferro,remove_inv,title,&
& symrel,tnons,symafm) ! Optional
    
 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_32_util
 use interfaces_42_geometry
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: natom,ntypat,nsym,timrev,space_group,npsp
 type(crystal_structure),intent(inout) :: Cryst
 logical,intent(in) :: remove_inv,use_antiferro
!arrays
 integer,intent(in) :: typat(natom)
 integer,optional,intent(in) :: symrel(3,3,nsym),symafm(nsym)
 real(dp),intent(in) :: xred(3,natom),rprimd(3,3),ziontypat(ntypat),znucl(npsp)
 real(dp),optional,intent(in) :: tnons(3,nsym)
 character(len=*),intent(in) :: title(ntypat) 

!Local variables-------------------------------
!scalars
 integer :: iat,indx,itypat,pinv,isym,nsym_noI
 real(dp) :: tolsym8,ucvol
 character(len=500) :: msg      
!arrays
 integer :: symrec(3,3),inversion(3,3)
 real(dp) :: gprimd(3,3),gmet(3,3),rmet(3,3)
 real(dp) :: spinrot(4)
 integer,pointer :: symrel_noI(:,:,:)
 integer,allocatable :: indsym(:,:,:)
 real(dp),pointer :: tnons_noI(:,:)
! *************************************************************************

 !@crystal_structure
 call nullify_crystal(Cryst)

 Cryst%natom  = natom 
 Cryst%ntypat = ntypat
 Cryst%npsp   = npsp

 Cryst%space_group = space_group

 call metric(gmet,gprimd,-1,rmet,rprimd,ucvol)
 
 Cryst%ucvol  = ucvol
 Cryst%rprimd = rprimd 
 Cryst%rmet   = rmet
 Cryst%gmet   = gmet
 Cryst%gprimd = gprimd

 Cryst%angdeg(1)=ACOS(Cryst%rmet(2,3)/SQRT(Cryst%rmet(2,2)*Cryst%rmet(3,3)))/two_pi*360.0d0
 Cryst%angdeg(2)=ACOS(Cryst%rmet(1,3)/SQRT(Cryst%rmet(1,1)*Cryst%rmet(3,3)))/two_pi*360.0d0
 Cryst%angdeg(3)=ACOS(Cryst%rmet(1,2)/SQRT(Cryst%rmet(1,1)*Cryst%rmet(2,2)))/two_pi*360.0d0

 allocate(Cryst%typat(natom),Cryst%xred(3,natom),Cryst%xcart(3,natom)) 
 allocate(Cryst%ziontypat(ntypat))
 allocate(Cryst%znucl(npsp))

 Cryst%typat     = typat 
 Cryst%xred      = xred 
 Cryst%ziontypat = ziontypat
 Cryst%znucl     = znucl

 call xredxcart(natom,1,rprimd,Cryst%xcart,Cryst%xred)

 allocate(Cryst%title(ntypat))
 Cryst%title=title
 !
 ! === Generate index table of atoms, in order for them to be used type after type ===
 allocate(Cryst%atindx(natom),Cryst%atindx1(natom))
 allocate(Cryst%nattyp(ntypat))

 indx=1
 do itypat=1,ntypat
  Cryst%nattyp(itypat)=0
  do iat=1,natom
   if (Cryst%typat(iat)==itypat) then
    Cryst%atindx (iat )=indx 
    Cryst%atindx1(indx)=iat
    indx=indx+1
    Cryst%nattyp(itypat)=Cryst%nattyp(itypat)+1
   end if
  end do
 end do

 Cryst%timrev = timrev
 call set2unit(inversion) ; inversion=-inversion

 if (PRESENT(symrel).and.PRESENT(tnons).and.PRESENT(symafm)) then 
  if (.not.remove_inv) then
   ! * Just a copy 
   Cryst%nsym= nsym
   allocate(Cryst%symrel(3,3,nsym),Cryst%symrec(3,3,nsym))
   allocate(Cryst%tnons(3,nsym),Cryst%symafm(nsym))
   Cryst%symrel=symrel 
   Cryst%tnons=tnons
   Cryst%symafm=symafm
   Cryst%use_antiferro = use_antiferro
   Cryst%has_inversion=.FALSE.
   do isym=1,nsym
    call mati3inv(symrel(:,:,isym),symrec)
    Cryst%symrec(:,:,isym)=symrec
    if (ALL(symrel(:,:,isym)==inversion)) Cryst%has_inversion=.TRUE. 
   end do
  else 
   ! * Remove inversion, just to be compatible with old GW implementation
   ! TODO should be removed!
   call remove_inversion(nsym,symrel,tnons,nsym_noI,symrel_noI,tnons_noI,pinv)
   Cryst%nsym=nsym_noI
   allocate(Cryst%symrel(3,3,nsym_noI),Cryst%symrec(3,3,nsym_noI))
   allocate(Cryst%tnons(3,nsym_noI),Cryst%symafm(nsym_noI))
   Cryst%symrel=symrel_noI
   Cryst%tnons=tnons_noI
   Cryst%has_inversion=.FALSE.
   if (ANY(symafm==-1)) then 
    msg = ' Solve the problem with inversion before adding ferromagnetic symmetries '
    MSG_BUG(msg)
   end if
   Cryst%symafm=1
   Cryst%use_antiferro=use_antiferro 
   do isym=1,nsym_noI
    call mati3inv(symrel_noI(:,:,isym),symrec)
    Cryst%symrec(:,:,isym)=symrec
   end do
   deallocate(symrel_noI,tnons_noI)
  end if

 else
  ! * Find symmetries symrec,symrel,tnons,symafm
  ! TODO This should be a wrapper around the abinit library whose usage is not so straightforward
  MSG_BUG('not yet implemented')
 end if

 ! === Obtain a list of rotated atoms ===
 ! $ R^{-1} (xred(:,iat)-\tau) = xred(:,iat_sym) + R_0 $ 
 ! * indsym(4,  isym,iat) gives iat_sym in the original unit cell.
 ! * indsym(1:3,isym,iat) gives the lattice vector $R_0$.
 ! 
 allocate(indsym(4,Cryst%nsym,natom))
 tolsym8=tol8
 call symatm(indsym,natom,Cryst%nsym,Cryst%symrec,Cryst%tnons,tolsym8,Cryst%typat,Cryst%xred)

 allocate(Cryst%indsym(4,Cryst%nsym,natom))
 Cryst%indsym=indsym  
 deallocate(indsym)

 ! === Rotation in spinor space ===
 allocate(Cryst%spinrot(4,Cryst%nsym))
 do isym=1,Cryst%nsym
  call getspinrot(Cryst%rprimd,spinrot,Cryst%symrel(:,:,isym))
  Cryst%spinrot(:,isym)=spinrot(:)
 end do

end subroutine init_crystal
!!***

!----------------------------------------------------------------------

!!****f* m_crystal/init_crystal_from_hdr 
!! NAME
!!  init_crystal_from_hdr
!!
!! FUNCTION
!!  Initializes a crystal_structure data type starting from the abinit header.
!!
!! INPUTS
!!  Hdr<Hdr_type>=the abinit header
!!  timrev ==2 => take advantage of time-reversal symmetry
!!         ==1 ==> do not use time-reversal symmetry 
!!  remove_inv [optional]= if .TRUE. the inversion symmetry is removed from the set of operations
!!  even though it is present in the header
!!
!! OUTPUT
!!  Cryst<crystal_structure>= the data type filled with data reported in the abinit header 
!!
!! TODO
!!  Add information on the use of time-reversal in the Abinit header.
!!
!! PARENTS
!!      berryphase_new,m_sigma_results,mlwfovlp_qp,mrgscr,rdm
!!      setup_bethe_salpeter,setup_screening,setup_sigma
!!
!! CHILDREN
!!      atmdata,etsf_io_data_write,etsf_io_low_close,etsf_io_low_def_var
!!      etsf_io_low_error_to_str,etsf_io_low_open_modify
!!      etsf_io_low_set_write_mode,etsf_io_low_write_dim,etsf_io_low_write_var
!!      wrtout
!!
!! SOURCE

subroutine init_crystal_from_hdr(Cryst,Hdr,timrev,remove_inv)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(Hdr_type),intent(in) :: Hdr
 type(crystal_structure),intent(out) :: Cryst 
 integer,intent(in) :: timrev
 logical,optional,intent(in) :: remove_inv

!Local variables-------------------------------
 integer :: space_group
 logical :: rinv,ltest,use_antiferro
! *********************************************************************

 rinv=.FALSE.; if (PRESENT(remove_inv)) rinv=remove_inv
 use_antiferro=(Hdr%nspden==2.and.Hdr%nsppol==1)

 ! === consistency check ===
 ltest = (timrev==1.or.timrev==2)
 ABI_CHECK(ltest,"Wrong value for timrev (1|2)")
 if (use_antiferro) then
  ABI_CHECK(ANY(Hdr%symafm==-1),"Wrong nspden, nsppol, symafm.")
 end if

 space_group=0 !FIXME not known

 call init_crystal(Cryst,space_group,Hdr%natom,Hdr%npsp,Hdr%ntypat,Hdr%nsym,Hdr%rprimd,Hdr%typat,Hdr%xred,&
& Hdr%zionpsp,Hdr%znuclpsp,timrev,use_antiferro,rinv,Hdr%title,&
& Hdr%symrel,Hdr%tnons,Hdr%symafm) ! Optional

end subroutine init_crystal_from_hdr
!!***

!----------------------------------------------------------------------

!!****f* m_crystal/nullify_crystal
!! NAME
!! nullify_crystal
!!
!! FUNCTION
!!  Nullify the pointers in the crystal_structure data type.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      m_crystal
!!
!! CHILDREN
!!      atmdata,etsf_io_data_write,etsf_io_low_close,etsf_io_low_def_var
!!      etsf_io_low_error_to_str,etsf_io_low_open_modify
!!      etsf_io_low_set_write_mode,etsf_io_low_write_dim,etsf_io_low_write_var
!!      wrtout
!!
!! SOURCE

subroutine nullify_crystal(Cryst)

 use defs_basis

 implicit none

!Arguments ------------------------------------
 type(crystal_structure),intent(inout) :: Cryst
! *********************************************************************

 !@crystal_structure

! integer
 nullify(Cryst%indsym )
 nullify(Cryst%symafm )
 nullify(Cryst%symrec )
 nullify(Cryst%symrel )
 nullify(Cryst%atindx )   
 nullify(Cryst%atindx1)   
 nullify(Cryst%typat  )   
 nullify(Cryst%nattyp )   

! real
 nullify(Cryst%tnons    )
 nullify(Cryst%xcart    )
 nullify(Cryst%xred     )
 nullify(Cryst%ziontypat)
 nullify(Cryst%znucl    )
 nullify(Cryst%spinrot  )

!character
 nullify(Cryst%title)

end subroutine nullify_crystal
!!***

!----------------------------------------------------------------------

!!****f* m_crystal/destroy_crystal
!! NAME
!!  destroy_crystal 
!!
!! FUNCTION
!!  Destroy the dynamic arrays in a Crystal_structure data type.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      berryphase_new,bethe_salpeter,m_gwannier,mlwfovlp_qp,mrgscr,rdm
!!      screening,sigma,vtorho
!!
!! CHILDREN
!!      atmdata,etsf_io_data_write,etsf_io_low_close,etsf_io_low_def_var
!!      etsf_io_low_error_to_str,etsf_io_low_open_modify
!!      etsf_io_low_set_write_mode,etsf_io_low_write_dim,etsf_io_low_write_var
!!      wrtout
!!
!! SOURCE

subroutine destroy_crystal(Cryst)

 use defs_basis

 implicit none

!Arguments ------------------------------------
 type(crystal_structure),intent(inout) :: Cryst

! *********************************************************************

 DBG_ENTER("COLL")
 
 !@crystal_structure

!integer
 if (associated(Cryst%indsym )) deallocate(Cryst%indsym )   
 if (associated(Cryst%symafm )) deallocate(Cryst%symafm )   
 if (associated(Cryst%symrec )) deallocate(Cryst%symrec )   
 if (associated(Cryst%symrel )) deallocate(Cryst%symrel )   
 if (associated(Cryst%atindx )) deallocate(Cryst%atindx )   
 if (associated(Cryst%atindx1)) deallocate(Cryst%atindx1)   
 if (associated(Cryst%typat  )) deallocate(Cryst%typat  )   
 if (associated(Cryst%nattyp )) deallocate(Cryst%nattyp )   

!real
 if (associated(Cryst%tnons    )) deallocate(Cryst%tnons    )   
 if (associated(Cryst%xcart    )) deallocate(Cryst%xcart    )   
 if (associated(Cryst%xred     )) deallocate(Cryst%xred     )   
 if (associated(Cryst%ziontypat)) deallocate(Cryst%ziontypat)
 if (associated(Cryst%znucl    )) deallocate(Cryst%znucl    )
 if (associated(Cryst%spinrot  )) deallocate(Cryst%spinrot  )   

!character
 if (associated(Cryst%title)) deallocate(Cryst%title    )

 DBG_EXIT("COLL")

end subroutine destroy_crystal
!!***

!----------------------------------------------------------------------

!!****f* m_crystal/print_crystal
!! NAME
!!  print_crystal 
!!
!! FUNCTION
!!  Print the content of crystal_structure data type
!!
!! INPUTS
!!  Cryst<crystal_structure>=The structure.
!!  [unit]=Unit number for output
!!  [prtvol]=Verbosity level
!!  [mode_paral]=Either "COLL" or "PERS"
!!  [header]=String to be printed as header for additional info.
!!
!! OUTPUT
!!  Only printing 
!!
!! PARENTS
!!      rdm,setup_bethe_salpeter,setup_screening,setup_sigma
!!
!! CHILDREN
!!      atmdata,etsf_io_data_write,etsf_io_low_close,etsf_io_low_def_var
!!      etsf_io_low_error_to_str,etsf_io_low_open_modify
!!      etsf_io_low_set_write_mode,etsf_io_low_write_dim,etsf_io_low_write_var
!!      wrtout
!!
!! SOURCE

subroutine print_crystal(Cryst,header,unit,mode_paral,prtvol) 

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14_hidewrite
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,optional,intent(in) :: unit,prtvol
 character(len=4),optional,intent(in) :: mode_paral 
 character(len=*),optional,intent(in) :: header
 type(crystal_structure),intent(in) :: Cryst

!Local variables-------------------------------
 integer :: my_unt,my_prtvol,nu
 character(len=4) :: my_mode
 character(len=500) :: msg      
! ********************************************************************* 

 my_unt   =std_out; if (PRESENT(unit      )) my_unt   =unit
 my_prtvol=0      ; if (PRESENT(prtvol    )) my_prtvol=prtvol 
 my_mode  ='COLL' ; if (PRESENT(mode_paral)) my_mode  =mode_paral

 msg=' ==== Info on the Cryst% object ==== '
 if (PRESENT(header)) msg=' ==== '//TRIM(ADJUSTL(header))//' ==== '
 call wrtout(my_unt,msg,my_mode)

 write(msg,'(a)')' Real(R)+Recip(G) space primitive vectors, cartesian coordinates (Bohr,Bohr^-1):'
 call wrtout(my_unt,msg,my_mode)
 do nu=1,3
  write(msg,'(1x,a,i1,a,3f11.7,2x,a,i1,a,3f11.7)')&
&  'R(',nu,')=',Cryst%rprimd(:,nu)+tol10,&
&  'G(',nu,')=',Cryst%gprimd(:,nu)+tol10 !tol10 is used to be consistent with metric.F90
  call wrtout(my_unt,msg,my_mode)
 end do

 write(msg,'(a,1p,e15.7,a)')&
& ' Unit cell volume ucvol=',Cryst%ucvol+tol10,' bohr^3'
 call wrtout(my_unt,msg,my_mode)

 write(msg,'(a,3es16.8,a)')&
& ' Angles (23,13,12)=',Cryst%angdeg(1:3),' degrees'
 call wrtout(my_unt,msg,my_mode)

 if (Cryst%timrev==1) then 
  write(msg,'(a)')' Time-reversal symmetry is not present '
 else if (Cryst%timrev==2) then 
  write(msg,'(a)')' Time-reversal symmetry is present '
 else 
  MSG_BUG('Wrong value for timrev') 
 end if
 call wrtout(my_unt,msg,my_mode)

 !if (Cryst%use_antiferro) then 
 ! write(msg,'(a)')' System has magnetic symmetries '
 ! call wrtout(my_unt,msg,my_mode)
 !end if

 call print_symmetries(Cryst%nsym,Cryst%symrel,Cryst%tnons,Cryst%symafm,unit=my_unt,mode_paral=my_mode)

end subroutine print_crystal
!!***

!----------------------------------------------------------------------

!!****f* m_crystal/print_symmetries
!! NAME
!! print_symmetries
!!
!! FUNCTION
!!  Helper function to print the set of symmetries.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!      hdr_vs_dtset,m_crystal
!!
!! CHILDREN
!!      atmdata,etsf_io_data_write,etsf_io_low_close,etsf_io_low_def_var
!!      etsf_io_low_error_to_str,etsf_io_low_open_modify
!!      etsf_io_low_set_write_mode,etsf_io_low_write_dim,etsf_io_low_write_var
!!      wrtout
!!
!! SOURCE

subroutine print_symmetries(nsym,symrel,tnons,symafm,unit,mode_paral)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14_hidewrite
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nsym
 integer,optional,intent(in) :: unit
 character(len=4),optional,intent(in) :: mode_paral
!arrays
 integer,intent(in) :: symrel(3,3,nsym),symafm(nsym)
 real(dp),intent(in) :: tnons(3,nsym)

!Local variables-------------------------------
 integer :: my_unt,isym,isymin,isymend,ii,jj
 character(len=500) :: msg      
 character(len=4) :: my_mode
! *********************************************************************

 my_unt =std_out; if (PRESENT(unit      )) my_unt =unit
 my_mode='COLL' ; if (PRESENT(mode_paral)) my_mode=mode_paral

 !write(msg,'(2a)')ch10,' Rotations                           Translations     Symafm '
 !do isym=1,nsym
 ! write(msg,'(1x,3(3i3,1x),4x,3(f11.7,1x),6x,i2)')symrel(:,:,isym),tnons(:,isym),symafm(isym)
 ! call wrtout(my_unt,msg,my_mode)
 !end do 

 write(msg,'(2a)')ch10,' Symmetry operations in real space (Rotation tnons AFM)'
 call wrtout(my_unt,msg,my_mode)
 do isymin=1,nsym,4
  isymend=isymin+3
  if (isymend>nsym) isymend=nsym
  do ii=1,3
   write(msg,'(4(3i3,f8.3,i3,3x))')((symrel(ii,jj,isym),jj=1,3),tnons(ii,isym),symafm(isym),isym=isymin,isymend)
   call wrtout(my_unt,msg,my_mode)
  enddo
  write(msg,'(a)')ch10
  call wrtout(my_unt,msg,my_mode)
 end do

end subroutine print_symmetries 
!!***

!----------------------------------------------------------------------

!!****f* m_crystal/abi_crystal_put
!! NAME
!! abi_crystal_put
!!
!! FUNCTION
!! Output system geometry to a file, using the NETCDF file format and ETSF I/O.
!! Data are taken from the crystal_structure object.
!!
!! INPUTS
!!  Cryst<crystal_structure>=Object defining the unit cell and its symmetries.
!!     %natom=Number of atoms in the unit cell
!!     %ntypat=Number of types of atoms in the unit cell.
!!     %npsp=Number of pseudopotentials.
!!     %space_group=Space group identifier.
!!     %typat(natom)=Type of each atom in the unit cell.
!!     %ziontypat(ntypat)=Charge of the pseudo-ion (No of valence electrons needed to screen exactly the pseudopotential).
!!     %znucl(ntypat)=Nuclear charge for each type of pseudopotential.
!!     %title(npsps)=The content of first line read from the psp file.
!!     %rprimd(3,3)=Real space dimensional primitive translations (bohr)
!!     %xred(3,natom)=Reduced coordinates of atoms.
!!     %symrel=Symmetry operations in real space (in terms of rprimd)
!!     %tnons(3,nsym)=Fractional translations.
!!     %symafm(nsym)=(Anti)ferromagnetic symmetries.
!!  fname=character string giving the root to form the name of the GEO file
!!
!! OUTPUT
!!  Data written in file whose name is fname.
!!
!! NOTES
!!  alchemy not treated, Crystal should be correctly Initialized at the beginning of the run.
!!
!! PARENTS
!!      m_sigma_results
!!
!! CHILDREN
!!      atmdata,etsf_io_data_write,etsf_io_low_close,etsf_io_low_def_var
!!      etsf_io_low_error_to_str,etsf_io_low_open_modify
!!      etsf_io_low_set_write_mode,etsf_io_low_write_dim,etsf_io_low_write_var
!!      wrtout
!!
!! SOURCE

subroutine abi_crystal_put(Cryst,fname)

 use defs_basis

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14_hidewrite
 use interfaces_32_util
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 character(len=fnlen),intent(in) :: fname
 type(crystal_structure),target,intent(in) :: Cryst
!arrays

!Local variables-------------------------------
!scalars
 integer :: itypat,ungeo
 integer,target :: space_group
 real(dp) :: amu,rcov
 logical :: lstat,ltest
 character(len=500) :: msg
#if defined HAVE_ETSF_IO
 character(len=etsf_io_low_error_len) :: errmess
 character(len=etsf_charlen) :: symmorphic
 !type(ETSF_dims) :: Dims
 type(ETSF_groups) :: Group_folder
 type(ETSF_geometry),target :: Geo_folder
 type(ETSF_io_low_error) :: Error_data
#endif
!arrays
 character(len=2),allocatable,target :: symbols(:)
 character(len=80),allocatable,target :: psp_desc(:),symbols_long(:)

! *************************************************************************

#if defined HAVE_ETSF_IO

 !@crystal_structure

 ! === Open the file ===
 write(msg,'(2a)')' abi_etsf_geo_put : about to open file ',TRIM(fname)
 call wrtout(std_out,msg,'COLL')

 call etsf_io_low_open_modify(ungeo,fname,lstat,Error_data=Error_data)
 if (.not.lstat) goto 1000

 ! === Define subset of ETSF-Dimensions and write them on file ===
 ! * Use low level procedures to write dimensions, since 
 !   etsf_io_dims_def correctly complains that some ETSF dims are missing.
 ! * If dimensions already exist, check that definitions are coherent.
 !Dims%number_of_atoms               = Cryst%natom
 !Dims%number_of_atom_species        = Cryst%ntypat
 !Dims%number_of_symmetry_operations = Cryst%nsym
 !call etsf_io_dims_def(ungeo,Dims,lstat,Error_data)
 !if (.not.lstat) goto 1000

 call etsf_io_low_write_dim(ungeo,'number_of_atoms',Cryst%natom,lstat,Error_data=Error_data)  
 if (.not.lstat) goto 1000  
 call etsf_io_low_write_dim(ungeo,'number_of_atoms_species',Cryst%ntypat,lstat,Error_data=Error_data)  
 if (.not.lstat) goto 1000  
 call etsf_io_low_write_dim(ungeo,'number_of_symmetry_operations',Cryst%nsym,lstat,Error_data=Error_data)  
 if (.not.lstat) goto 1000  

 ! === Close the file, due to call to etsf_io_data_write  ===
 call etsf_io_low_close(ungeo,lstat,Error_data=Error_data)
 if (.not.lstat) goto 1000

 ! === Fill-in ETSF geometry folder ===

 ! FIXME alchemy not treated since Cryst should be initialized in invars2
 ltest=(Cryst%npsp==Cryst%ntypat)
 ABI_CHECK(ltest,"alchemy not supported")

 ! === Set-up atomic symbols ===
 allocate(symbols(Cryst%ntypat),symbols_long(Cryst%ntypat),psp_desc(Cryst%ntypat))
 do itypat=1,Cryst%ntypat  
  call atmdata(amu,rcov,symbols(itypat),Cryst%znucl(itypat)) 
  write(symbols_long(itypat),'(a2,a78)') symbols(itypat),REPEAT(CHAR(0),78)
  write(psp_desc(itypat),'(2a)') &
&  Cryst%title(itypat)(1:MIN(80,LEN_TRIM(Cryst%title(itypat)))),REPEAT(CHAR(0),MAX(0,80-LEN_TRIM(Cryst%title(itypat))))
 end do

 space_group=0; if (Cryst%space_group>0) space_group=Cryst%space_group
 Geo_folder%space_group                   => space_group
 Geo_folder%primitive_vectors             => Cryst%rprimd
 Geo_folder%reduced_symmetry_matrices     => Cryst%symrel
 Geo_folder%reduced_symmetry_translations => Cryst%tnons
 Geo_folder%atom_species                  => Cryst%typat
 Geo_folder%reduced_atom_positions        => Cryst%xred
 if (Cryst%npsp==Cryst%ntypat) then
  Geo_folder%valence_charges              => Cryst%ziontypat
 end if
 Geo_folder%atomic_numbers                => Cryst%znucl
 Geo_folder%atom_species_names            => symbols_long
 Geo_folder%chemical_symbols              => symbols
 Geo_folder%pseudopotential_types         => psp_desc

 symmorphic='no'; if (isymmorphic(Cryst)) symmorphic='yes'

 Group_folder%geometry => Geo_folder

 call etsf_io_data_write(fname,Group_folder,lstat,Error_data)
 if (.not.lstat) goto 1000

 deallocate(symbols,symbols_long,psp_desc)
 !
 ! === At this point we have an ETSF-compliant file ===
 ! * Add additional stuff for internal use in abinit.
 ! TODO add spinat.

 call etsf_io_low_open_modify(ungeo,fname,lstat,Error_data=Error_data)
 if (.not.lstat) goto 1000

 call etsf_io_low_def_var(ungeo,'symafm',etsf_io_low_integer,&
& (/'number_of_symmetry_operations'/),lstat,Error_data=Error_data)
 if (.not.lstat) goto 1000

 call etsf_io_low_set_write_mode(ungeo,lstat,Error_data=Error_data)
 if (.not.lstat) goto 1000

 call etsf_io_low_write_var(ungeo,'symafm',Cryst%symafm,lstat,Error_data=Error_data)
 if (.not.lstat) goto 1000

 call etsf_io_low_close(ungeo,lstat,Error_data=Error_data)
 if (.not.lstat) goto 1000

 ! === Handle the error ===
 1000 continue
 if (.not.lstat) then
  call etsf_io_low_error_to_str(errmess,Error_data)
  msg = errmess(1:min(500,len(errmess)))
  MSG_ERROR(msg)
 end if

#else
 msg = 'ETSF-IO support is not activated.'
 MSG_ERROR(msg)
#endif

end subroutine abi_crystal_put
!!***

!----------------------------------------------------------------------

!!****f* m_crystal/idx_spatial_inversion
!! NAME
!!  idx_spatial_inversion
!!
!! FUNCTION
!!  Return the index of the spatial inversion, 0 if not present
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

function idx_spatial_inversion(Cryst) result(inv_idx)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 integer :: inv_idx
 type(crystal_structure),intent(in) :: Cryst

!Local variables-------------------------------
!scalars
 integer :: isym
!arrays
 integer :: inversion(3,3)

! *************************************************************************

 inversion=RESHAPE((/-1,0,0,0,-1,0,0,0,-1/),(/3,3/))

 inv_idx=0
 do isym=1,Cryst%nsym
  if ( ALL(Cryst%symrel(:,:,isym)==inversion) ) then 
   inv_idx=isym
   RETURN
  end if
 end do

end function idx_spatial_inversion
!!***

!----------------------------------------------------------------------

!!****f* m_crystal/isymmorphic
!! NAME
!!  isymmorphic
!!
!! FUNCTION
!!  Returns .TRUE. is space group is symmorphic, i.e. all fractional translations are zero.
!!
!! INPUTS
!!
!! OUTPUT
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

function isymmorphic(Cryst) result(ans)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 logical :: ans
 type(crystal_structure),intent(in) :: Cryst

! *************************************************************************

 ans = ALL (ABS(Cryst%tnons)<tol6) 

end function isymmorphic

!----------------------------------------------------------------------

END MODULE m_crystal
!!***
