!{\src2tex{textfont=tt}}
!!****f* ABINIT/complete_gamma
!!
!! NAME
!! complete_gamma
!!
!! FUNCTION
!! Use the set of special q points calculated by the Monkhorst &
!! Pack Technique.
!! Check if all the informations for the q points are present in
!! the input gamma matrices.
!! Generate the gamma matrices (already summed over the FS) of the set of q points which
!! samples homogeneously the entire Brillouin zone.
!!
!! COPYRIGHT
!! Copyright (C) 2009-2010 ABINIT group (MVer)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!! elph_ds = datastructure for elphon information (mainly
!!   matrix elements and dimensions)
!!
!! gprimd(3,3)=dimensionful primitive translations in reciprocal space
!! indsym = map of atoms by inverses of symrels
!! natom=number of atoms in unit cell
!! nsym=number of space group symmetries
!! qpttoqpt = qpoint index mapping under symops
!! rprimd(3,3)=dimensionful primitive translations in real space
!! symrec(3,3,nsym)=3x3 matrices of the group symmetries (recip space)
!! symrel(3,3,nsym)=3x3 matrices of the group symmetries (real space)
!! tnons(3,nsym)=nonsymmorphic translations associated to symrel
!!
!! OUTPUT
!! gamma_qpt = in/out: set of gamma matrix elements completed and symmetrized 
!!    gamma_qpt(2,elph_ds%nbranch*elph_ds%nbranch,elph_ds%nsppol,elph_ds%nqpt_full)
!!
!! TODO
!!
!! PARENTS
!!
!! CHILDREN
!!      zgemm
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine complete_gamma(elph_ds,gamma_qpt,&
&   gprimd,indsym,natom,nsym,qpttoqpt,rprimd,&
&   symrec,symrel)

 use defs_basis
 use defs_elphon

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: natom,nsym
 type(elph_type),intent(inout) :: elph_ds
!arrays
 integer,intent(in) :: indsym(4,nsym,natom)
 integer,intent(in) :: qpttoqpt(2,nsym,elph_ds%nqpt_full),symrec(3,3,nsym)
 integer,intent(in) :: symrel(3,3,nsym)
 real(dp),intent(in) :: gprimd(3,3)
 real(dp),intent(in) :: rprimd(3,3)
 real(dp), intent(inout) :: gamma_qpt(2,elph_ds%nbranch*elph_ds%nbranch,elph_ds%nsppol,elph_ds%nqpt_full)

!Local variables-------------------------------
!scalars
 integer :: ibranch,ieqqpt,ii
 integer :: iqpt,isppol,isym
 integer :: itim,jbranch,jj,kk,ll
 integer :: neqqpt
 integer :: iatom,ancestor_iatom
 integer :: iqpt_fullbz

 real(dp),parameter :: tol=2.d-8
!arrays
 integer :: symmetrized_qpt(elph_ds%nqpt_full)
 integer :: gkk_flag(elph_ds%nbranch,elph_ds%nbranch,elph_ds%nsppol,elph_ds%nqpt_full)
 real(dp) :: ss(3,3)
 real(dp) :: tmp_mat(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: tmp_mat2(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp),allocatable :: gkk_qpt_new(:,:,:),gkk_qpt_tmp(:,:,:)

 real(dp) :: ss_allatoms(2,elph_ds%nbranch,elph_ds%nbranch)
 real(dp) :: c_one(2), c_zero(2)


! *********************************************************************


 c_one = (/one,zero/)
 c_zero = (/zero,zero/)

!Generation of the gkk matrices relative to the q points
!of the set which samples the entire Brillouin zone

!set up flags for gamma_qpt matrices we have
 gkk_flag = -1
 do iqpt=1,elph_ds%nqptirred
   iqpt_fullbz = elph_ds%qirredtofull(iqpt)
   gkk_flag(:,:,:,iqpt_fullbz) = 1
 end do

 symmetrized_qpt(:) = -1
 isppol=1

 allocate (gkk_qpt_new(2,elph_ds%nbranch*elph_ds%nbranch, elph_ds%nsppol))
 allocate (gkk_qpt_tmp(2,elph_ds%nbranch*elph_ds%nbranch, elph_ds%nsppol))

 do iqpt=1,elph_ds%nqpt_full

!  Already symmetrized?
   if (symmetrized_qpt(iqpt) == 1) cycle

   gkk_qpt_new(:,:,:) = zero

!  loop over qpoints equivalent to iqpt
   neqqpt=0
!  do not use time reversal symmetry to complete the qpoints:
!  do not know what happens to the gamma matrices
   itim=1

!  do itim=1,2
   do isym=1,nsym
!    ieqqpt is sent onto iqpt by itim/isym
     ieqqpt = qpttoqpt(itim,isym,iqpt)


     if (gkk_flag(1,1,1,ieqqpt) == -1) cycle
!    if we have information on this qpt
!    iqpt is equivalent to ieqqpt: get it from file or memory
     gkk_qpt_tmp(:,:,:) = gamma_qpt(:,:,:,ieqqpt)


     neqqpt=neqqpt+1

     if (elph_ds%ep_scalprod==1) then
       do ii=1,3
         do jj=1,3
           ss(ii,jj)=0.0_dp
           do kk=1,3
             do ll=1,3
               ss(ii,jj)=ss(ii,jj)+rprimd(ii,kk)*symrel(kk,ll,isym)*gprimd(ll,jj)
             end do
           end do
         end do
       end do
     else
       do ii=1,3
         do jj=1,3
           ss(ii,jj) = symrel(jj,ii,isym)
         end do
       end do
     end if

     ss_allatoms(:,:,:) = zero
     do iatom=1,natom
       ancestor_iatom = indsym(4,isym,iatom)
       ss_allatoms(1,(ancestor_iatom-1)*3+1:(ancestor_iatom-1)*3+3,&
&       (iatom-1)*3+1:         (iatom-1)*3+3) = ss(1:3,1:3)
     end do


!    NOTE   ssinv(ii,jj)=ssinv(ii,jj)+gprimd(ii,kk)*rprimd(jj,ll)*symrec(ll,kk,isym)

     do isppol=1,elph_ds%nsppol
!      multiply by the ss matrices
       tmp_mat2(:,:,:) = zero
       tmp_mat(:,:,:) = reshape(gkk_qpt_tmp(:,:,isppol),&
&       (/2,elph_ds%nbranch,elph_ds%nbranch/))
       call ZGEMM ('N','N',elph_ds%nbranch,elph_ds%nbranch,elph_ds%nbranch,&
&       c_one,ss_allatoms,elph_ds%nbranch,tmp_mat,elph_ds%nbranch,c_zero,&
&       tmp_mat2,elph_ds%nbranch)
       call ZGEMM ('N','T',elph_ds%nbranch,elph_ds%nbranch,elph_ds%nbranch,&
&       c_one,tmp_mat2,elph_ds%nbranch,ss_allatoms,elph_ds%nbranch,c_zero,&
&       tmp_mat,elph_ds%nbranch)

!      add to gkk_qpt_new
       do ibranch =1,elph_ds%nbranch
         do jbranch =1,elph_ds%nbranch
           gkk_qpt_new(:,(jbranch-1)*elph_ds%nbranch+ibranch,isppol) = &
&           gkk_qpt_new(:,(jbranch-1)*elph_ds%nbranch+ibranch,isppol) + &
&           tmp_mat(:,jbranch,ibranch)
         end do
       end do

     end do ! end isppol do

   end do ! end isym do

   if (neqqpt > 1) then
     write (*,*) ' found several equiv qpts and am symmetrizing them ', neqqpt
   end if

!  divide by number of equivalent qpts found
   gkk_qpt_new(:,:,:) = gkk_qpt_new(:,:,:)/neqqpt

!  copy the symmetrized version into all the equivalent qpoints, appropriately transformed
!  See above
   itim=1
!  do itim=1,2
   do isym=1,nsym
!    ieqqpt is sent onto iqpt by itim/isym
     ieqqpt = qpttoqpt(itim,isym,iqpt)

     if (symmetrized_qpt(ieqqpt) /= -1) cycle
     gkk_qpt_tmp(:,:,:) = zero

!    use symrec matrices to get inverse transform from isym^{-1}
     if (elph_ds%ep_scalprod==1) then
       do ii=1,3
         do jj=1,3
           ss(ii,jj)=0.0_dp
           do kk=1,3
             do ll=1,3
!              Use inverse of symop matrix here to get back to ieqqpt (inv+transpose is in symrec and in gprimd)
               ss(ii,jj)=ss(ii,jj)+rprimd(ii,kk)*symrec(ll,kk,isym)*gprimd(jj,ll)
             end do
           end do
         end do
       end do
     else
       do ii=1,3
         do jj=1,3
           ss(ii,jj) = symrec(ii,jj,isym)
         end do
       end do
     end if

     ss_allatoms(:,:,:) = zero
     do iatom=1,natom
       ancestor_iatom = indsym(4,isym,iatom)
       ss_allatoms(1,(ancestor_iatom-1)*3+1:(ancestor_iatom-1)*3+3,&
&       (iatom-1)*3+1:          (iatom-1)*3+3) = ss(1:3,1:3)
     end do

!    ! Use inverse of symop matrix here to get back to ieqqpt
!    ssinv(ii,jj)=ssinv(ii,jj)+gprimd(ii,kk)*rprimd(jj,ll)*symrel(kk,ll,isym)

     do isppol=1,elph_ds%nsppol
!      multiply by the ss^{-1} matrices
       tmp_mat2(:,:,:) = zero
       tmp_mat(:,:,:) = reshape(gkk_qpt_new(:,:,isppol),&
&       (/2,elph_ds%nbranch,elph_ds%nbranch/))
       call ZGEMM ('N','N',elph_ds%nbranch,elph_ds%nbranch,elph_ds%nbranch,&
&       c_one,ss_allatoms,elph_ds%nbranch,tmp_mat,elph_ds%nbranch,c_zero,&
&       tmp_mat2,elph_ds%nbranch)
       call ZGEMM ('N','T',elph_ds%nbranch,elph_ds%nbranch,elph_ds%nbranch,&
&       c_one,tmp_mat2,elph_ds%nbranch,ss_allatoms,elph_ds%nbranch,c_zero,&
&       tmp_mat,elph_ds%nbranch)

!      FIXME: the following could just be a reshape
       do ibranch =1,elph_ds%nbranch
         do jbranch =1,elph_ds%nbranch
           gkk_qpt_tmp(:,(jbranch-1)*elph_ds%nbranch+ibranch,isppol) =&
&           tmp_mat(:,jbranch,ibranch)
         end do
       end do

       if (gkk_flag (1,1,isppol,ieqqpt) == -1) then
         gkk_flag (:,:,isppol,ieqqpt) = 0
       end if

     end do ! end isppol do


!    save symmetrized matrices for qpt ieqqpt
     gamma_qpt(:,:,:,ieqqpt) = gkk_qpt_tmp(:,:,:)

     symmetrized_qpt(ieqqpt) = 1

   end do ! end isym do 
!  end do ! end itim do
 end do
!end iqpt do

end subroutine complete_gamma
!!***
