!{\src2tex{textfont=tt}}
!!****f* ABINIT/elphon
!!
!! NAME
!! elphon
!!
!! FUNCTION
!! This routine extracts the electron phonon coupling matrix
!! elements and calculates related properties - Tc, phonon linewidths...
!!
!! COPYRIGHT
!! Copyright (C) 2004-2010 ABINIT group (MVer, MG)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!   anaddb_dtset=dataset with input variables
!!     anaddb_dtset%a2fsmear = smearing for alpha2F function
!!     anaddb_dtset%brav = type of Bravais lattice
!!     anaddb_dtset%dipdip  =dipole dipole interaction flag
!!     anaddb_dtset%elphsmear = smearing width for gaussian integration
!!           or buffer in energy for calculations with tetrahedra (telphint=0)
!!     anaddb_dtset%elph_fermie = input value of Fermi energy
!!           0 means use value from wfk file
!!     anaddb_dtset%enunit = governs the units to be used for the output of
!!           the phonon frequencies and e-ph quantities
!!     anaddb_dtset%gkk2write= flag to write out gkk2 matrix elements to disk
!!     anaddb_dtset%gkk_rptwrite= flag to write out real space gkk_rpt matrix elements to disk
!!     anaddb_dtset%gkqwrite= flag to write out gkq matrix elements to disk
!!     anaddb_dtset%ep_b_min= first band taken into account in FS integration (if telphint==2)
!!     anaddb_dtset%ep_b_max= last band taken into account in FS integration (if telphint==2) 
!!     anaddb_dtset%prtfsurf = integer flag for the output of the Fermi surface (XCrysden file format)
!!     anaddb_dtset%prtnest = integer flag for the calculation of the nesting function
!!     anaddb_dtset%ifcflag = flag for IFC matrices in anaddb calling routine
!!           the IFCs are presumed to be known!
!!     anaddb_dtset%ifltransport= flag for transport properties (no=0: yes=1 )
!!     anaddb_dtset%kptrlatt=kpoint grid generating vectors, as in abinit
!!     anaddb_dtset%mustar = parameter for Coulombic pseudo-potential in McMillan T_c calculation
!!     anaddb_dtset%ngqpt(3)=integers defining the number of points in the qpt sampling
!!     anaddb_dtset%nqpath=number of vertices in the path in reciprocal space, for band structure
!!           and phonon linewidth output
!!     anaddb_dtset%nqshft= number of shift vectors for defining the sampling of q points
!!     anaddb_dtset%ntemper = number of temperature points to calculate, from tempermin to 
!!           tempermin+ntemper*temperinc
!!     anaddb_dtset%qpath=vertices in the path in reciprocal space, for band structure
!!           and phonon linewidth output
!!     anaddb_dtset%q1shft(3,4) =qpoint shifts considered
!!     anaddb_dtset%telphint = flag for integration over the FS with 0=tetrahedra 1=gaussians
!!     anaddb_dtset%tempermin = minimum temperature at which resistivity etc are calculated (in K)
!!     anaddb_dtset%temperinc = interval temperature grid on which resistivity etc are calculated (in K)
!!     anaddb_dtset%ep_keepbands = flag to keep gamma matrix dependence on electronic bands
!!
!!   acell_in(3)= input length scales of cell (bohr)
!!   amu(ntypat)=mass of the atoms (atomic mass unit)
!!   atmfrc  = inter-atomic force constants from anaddb
!!   dielt(3,3) = dielectric tensor
!!   dyewq0(3,3,natom)=atomic self-interaction correction to the
!!        dynamical matrix (only when anaddb_dtset%dipdip=1)
!!     or buffer in energy for calculations with tetrahedra (telphint=0)
!!     0 means use value from wfk file
!!   elph_base_name=base name for output files
!!   gmet(3,3) =metric in reciprocal space (telphint=1)
!!   gprim(3,3) =dimensionless basis vectors of reciprocal space
!!   indsym = mapping of atoms btw themselves under symmetry
!!   mpert =maximum number of ipert
!!   natom=number of atoms in cell
!!   nrpt =number of real space points used to integrate IFC (for
!!        interpolation of dynamical matrices)
!!   nsym=number of space group symmetries
!!   ntypat = number of types of atoms
!!   rcan(3,natom) =canonical positions of atoms
!!   rmet(3,3)=metric tensor in real space (bohr^2)
!!   rprim_in(3,3)= input primitive translation vectors
!!   rpt(3,nprt) =canonical positions of R points in the unit cell
!!   symrec(3,3,nsym)=3x3 matrices of the group symmetries (reciprocal space)
!!   symrel(3,3,nsym)=3x3 matrices of the group symmetries (real space)
!!   tnons(3,nsym)=fractional nonsymmorphic translations
!!   trans(3,natom) = Atomic translations : xred = rcan + trans
!!   typat(natom)=type integer for each atom in cell
!!   ucvol=unit cell volume in bohr**3
!!   unitgkk = fortran unit for file containing matrix elements, from mrggkk
!!   wghatm(natom,natom,nrpt) =Weight for the pair of atoms and the R vector
!!   xred(3,natom)=fractional dimensionless atomic coordinates
!!   zeff(3,3,natom) =effective charge on each atom, versus electric
!!        field and atomic displacement
!!
!! OUTPUT
!!
!! NOTES
!!  inspired to a large extent by epcouple.f from the DecAFT package by J. Kay Dewhurst
!!  most inputs taken from mkifc.f
!!  in anaddb anaddb_dtset%ifcflag must be 1 such that the IFC are calculated in atmfrc prior to calling elphon
!!
!!  brav not taken into account propely in all of the code. (MG?)
!!
!!  could choose to make a full 3 dimensional kpt array (:,:,:). Easier for many operations
!!
!! PARENTS
!!      anaddb
!!
!! CHILDREN
!!      clean_phon_ds,complete_gamma,eliashberg_1d,ep_setupqpt,ftgam
!!      get_all_gkk2,get_all_gkq,get_all_gkr,get_fs_kpts,get_gkk_qpt_tr
!!      get_tetra,get_tetra_weight,hdr_clean,integrate_gamma,integrate_gamma_tr
!!      leave_new,matr3inv,mka2f,mka2f_tr,mka2fqgrid,mkfskgrid,mknesting
!!      mkph_linwid,mkqptequiv,mkrdim,order_fs_kpts,outelph,printbxsf
!!      rchkgsheader,setup_phon_ds,symkpt,wrap2_pmhalf,wrtout
!!
!! NOTES
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine elphon(anaddb_dtset,acell_in,amu,atmfrc,&
&  ddkfilename,dielt,dyewq0,elph_base_name,gmet,&
&  gprim,indsym,mpert,mpi_enreg,natom,nrpt,nsym,ntypat,&
&  rcan,rmet,rprim_in,rpt,symrec,symrel,tnons,trans,typat,ucvol,&
&  unitgkk,wghatm,xred,zeff)

 use defs_basis
 use defs_datatypes
 use defs_abitypes
 use defs_elphon
 use m_header,          only : hdr_clean

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
 use interfaces_32_util
 use interfaces_42_geometry
 use interfaces_56_recipspace
 use interfaces_62_occeig
 use interfaces_77_ddb, except_this_one => elphon
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: mpert,natom
 integer,intent(in) :: nrpt,nsym,ntypat
 integer,intent(in) :: unitgkk
 real(dp),intent(in) :: ucvol
 character(len=fnlen) :: elph_base_name
 character(len=fnlen),intent(in) :: ddkfilename
 type(MPI_type),intent(inout) :: mpi_enreg
 type(anaddb_dataset_type) :: anaddb_dtset
!arrays
 integer,intent(in) :: indsym(4,nsym,natom)
 integer,intent(in) :: symrec(3,3,nsym),symrel(3,3,nsym),typat(natom)
 real(dp),intent(in) :: acell_in(3),amu(ntypat),atmfrc(2,3,natom,3,natom,nrpt)
 real(dp),intent(in) :: dielt(3,3)
 real(dp),intent(in) :: dyewq0(3,3,natom),gmet(3,3),gprim(3,3)
 real(dp),intent(in) :: rcan(3,natom),rmet(3,3),rprim_in(3,3)
 real(dp),intent(in) :: rpt(3,nrpt),tnons(3,nsym),trans(3,natom)
 real(dp),intent(in) :: wghatm(natom,natom,nrpt),xred(3,natom),zeff(3,3,natom)

!Local variables-------------------------------
!scalars
 integer :: iFSkpt,iatom,ib1
 integer :: iband,ieliash,ifermi,ii,ikpt
 integer :: iqpt,irpt,isppol,istat,isym
 integer :: maxFSband,minFSband,mtetra,n1wf,nFSkpt,nband
 integer :: neliash,nene,nqpt1
 integer :: nsegment,ntetra,onegkk2size,onegkksize
 integer :: option,timrev
 integer :: qtor
 real(dp) :: deltaene,elphsmear_tmp,enemax,enemin,factor,gaussfactor
 real(dp) :: gaussprefactor,max_occ,rcvol,realdp_ex
 real(dp) :: refdist,res,ss,tmpdist,vv,xx
 logical :: make_gkk2,use_afm
 character(len=500) :: message
 character(len=fnlen) :: fname
 type(elph_tr_type) :: elph_tr_ds
 type(elph_type) :: elph_ds
 type(hdr_type) :: hdr
 type(phon_type) :: phon_ds
!arrays
 integer :: qptrlatt(3,3)
 integer,allocatable :: FSfullpqtofull(:,:),FSfulltofull(:,:,:)
 integer,allocatable :: FSfulltoirred(:,:),FSirredtoGS(:),FSirredtofull(:)
 integer,allocatable :: FSkptflag(:),FSkptirrank(:),gkk_flag(:,:,:,:,:)
 integer,allocatable :: indqpt1(:),npoint(:),qpttoqpt(:,:,:),tetra_full(:,:,:)
 integer,allocatable :: tetra_mult(:),tetra_wrap(:,:,:),tmpFSfulltofull(:,:,:)
 integer,allocatable :: tmpFSfulltoirred(:,:)
 real(dp) :: acell(3)
 real(dp) :: ftwghtgkk(natom,nrpt),gprimd(3,3)
 real(dp) :: klatt(3,3),kpt(3),rlatt(3,3),rprim(3,3)
 real(dp) :: rprimd(3,3),shiftk(3),tmpkpt(3)
 real(dp),allocatable :: FSirredwtk(:),FSkpt(:,:),FSkptirred(:,:)
 real(dp),allocatable :: a2f_1d(:),delta(:,:),dos_phon(:)
 real(dp),allocatable :: dtweightde(:,:),dummy_eig(:,:,:)
 real(dp),allocatable :: eigenGS(:,:,:)
 real(dp),allocatable :: qptirred(:,:)
 real(dp),allocatable :: tmpFSkpt(:,:),tmp_eigen(:)
 real(dp),allocatable :: tweight(:,:),wtq(:),wtq_folded(:)
 real(dp),allocatable :: zz(:,:)

! *************************************************************************

!==================================
!Initialization of some variables
!==================================

 neliash = 10                     !maximum number of iterations to converge Tc
 elph_ds%mustar=anaddb_dtset%mustar            !input mustar
 elph_ds%nbranch = 3*natom        !number of phonon modes = 3 * natom
 elph_ds%ep_keepbands = anaddb_dtset%ep_keepbands  !flag to sum over bands
 elph_ds%a2fsmear = anaddb_dtset%a2fsmear   !smearing for Eliashber functions

 elph_ds%tuniformgrid = 1

 elph_ds%na2f = 400               !maximum number of Matsubara frequencies.
!The precise number used depends on the value of Tc:
!they span $w_n = (2n+1) \pi T_c$  where $abs(w_n) < w_{cutoff}$
!ie $|n| < n_{cutoff} = ( \frac{w_{cutoff}}{\pi T_c} ) / 2$
 write (*,*) ' elphon : na2f = ', elph_ds%na2f
 
!use time reversal symmetry always when possible for kpoint reduction,
!and suppose it has been used in WF generation
!not used for the moment: values are always taken from input files.
 timrev = 1

!save gkk data for full kpoints to file on disk

 elph_ds%gkqwrite=anaddb_dtset%gkqwrite
 elph_ds%gkk_rptwrite=anaddb_dtset%gkk_rptwrite
 elph_ds%gkk2write=anaddb_dtset%gkk2write

!This should never be turned off: symmetrization of elphon matrix elements
!in complete_gkk. See get_all_gkq
 elph_ds%symgkq=anaddb_dtset%symgkq

 elph_ds%elph_base_name = trim(elph_base_name)

!normalize input rprim and acell.
 do ii=1,3
   ss = sqrt(rprim_in(1,ii)**2+rprim_in(2,ii)**2+rprim_in(3,ii)**2)
   rprim(:,ii) = rprim_in(:,ii)/ss
   acell(ii) = acell_in(ii) * ss
 end do

!make dimension-ful rprimd and gprimd for transformation of derivatives to cartesian coordinates.
 call mkrdim(acell,rprim,rprimd)
 call matr3inv(rprimd,gprimd)

!===================
!Check some inputs
!===================

 if (nsym==1) then
   write (message,'(7a)')ch10,&
&   ' elphon: COMMENT- ',ch10,&
&   ' Symmetries are not used! ',ch10,&
&   ' Full matrix elements must be supplied for all perturbations and qpoints!',ch10
   call wrtout(std_out,message,'COLL')
   call wrtout(ab_out,message,'COLL')
   
   if (abs(tnons(1,1))+abs(tnons(2,1))+abs(tnons(3,1)) > tol10) then
     write (message,'(4a)')ch10,&
&     ' elphon : ERROR-',ch10,&
&     ' tnons should be (0,0,0) for unique symmetry Id'
     call wrtout(std_out,message,'COLL')
     call leave_new('COLL')
   end if
 end if

 if (anaddb_dtset%ifcflag/=1) then
   write(message,'(6a,i4)')ch10,&
&   ' elphon : ERROR-',ch10,&
&   ' ifcflag should be set to one,',ch10,&
&   ' the IFC matrices are supposed to exist.',anaddb_dtset%ifcflag
   call wrtout(std_out,message,'COLL')
   call wrtout(ab_out,message,'COLL')
 end if

!=================================
!Set up the full grid of qpoints
!=================================
 call ep_setupqpt (anaddb_dtset,elph_ds,nsym,qptrlatt,rprimd,symrel)

!=================================================================
!Calculate weights, needed to estimate lambda using the weighted
!sum of the uninterpolated e-ph matrix elements
!=================================================================

 write (message,'(a)')' setqgrid : calling symkpt to find irred q points'
 call wrtout(6,message,'COLL')

 allocate(indqpt1(elph_ds%nqpt_full),wtq_folded(elph_ds%nqpt_full),wtq(elph_ds%nqpt_full),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating indqpt1 ,wtq_folded ,wtq'
 wtq(:) = one/dble(elph_ds%nqpt_full) !weights normalized to unity

!
!NOTE: this reduction of irred qpt may not be identical to that in GKK file
!which would be more practical to use.
!
 option=0 !do not write to ab_out
!should we save indqpt1 for use inside elph_ds?
 call symkpt(gmet,indqpt1,elph_ds%qpt_full,elph_ds%nqpt_full,nqpt1,nsym,option,symrec,&
& timrev,wtq,wtq_folded)

 write (message,'(2a,i6)')ch10,' Number of irreducible q-points = ',nqpt1
 call wrtout(6,message,'COLL')
 elph_ds%nqptirred=nqpt1

 write (message,'(a)')' Irreducible q points with weights :'
 call wrtout(6,message,'COLL')

 do iqpt=1,elph_ds%nqpt_full
   if (wtq_folded(iqpt) /= zero) then
     write (message,'(1x,i4,a2,4es16.8)')iqpt,') ',elph_ds%qpt_full(:,iqpt),wtq_folded(iqpt)
     call wrtout(6,message,'COLL')
   end if
 end do

 write(message,'(a)')ch10
 call wrtout(6,message,'COLL')

 allocate (elph_ds%wtq(elph_ds%nqpt_full),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating elph_ds%wtq'
 elph_ds%wtq(:)=wtq_folded(:)
!MEMO indqpt could be useful to test the qgrid read by abinit
 deallocate (indqpt1,wtq_folded,wtq)

!================================================================
!Summed weights for gkk FT. Is wghatm symmetric on the 2 atoms?
!present version does not use ftwghtgkk
!================================================================
 do iatom=1,natom
   do irpt=1,nrpt
     ftwghtgkk(iatom,irpt) = wghatm(iatom,iatom,irpt)
   end do
 end do

!====================================
!Read the GS header of the GKK file
!====================================
 
 write (message,'(2a)')ch10,' elphon : reading and checking the GS header of the GKK file'
 call wrtout (std_out,message,'COLL')

 call rchkGSheader(hdr,natom,nband,unitgkk)

 elph_ds%nsppol=hdr%nsppol
 elph_ds%nspinor=hdr%nspinor

!Set elph_ds%fermie: either comes from anaddb input file or from wfk file
 elph_ds%fermie = hdr%fermie
 if (abs(anaddb_dtset%elph_fermie) > tol10) then
   elph_ds%fermie = anaddb_dtset%elph_fermie
   write(message,'(a,f12.6)')' Fermi level set by the user at :',elph_ds%fermie
   call wrtout(std_out,message,'COLL')
 end if

 do iatom=1,natom
   do irpt=1,nrpt
     ftwghtgkk(iatom,irpt) = wghatm(iatom,iatom,irpt)
   end do
 end do

!==================================================
!Read GS eigenvalues for each irreducible kpt and
!number of 1WF files contributing to the GKK file
!==================================================

!allocate arrays which depend on general variables
 allocate(eigenGS(nband,hdr%nkpt,elph_ds%nsppol),FSkptflag(hdr%nkpt),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating eigenGS FSkptflag'
 do isppol=1,elph_ds%nsppol
   do ikpt=1,hdr%nkpt
     read(unitgkk) eigenGS(:,ikpt,isppol)
   end do
 end do

!read number of 1WF files contributing to the GKK file
 read(unitgkk) n1wf
 write(message,'(a,i4)')' elphon : number of perturbations in the gkk file = ',n1wf
 call wrtout(std_out,message,'COLL')

!====================================================================
!Setup of the kgrid :   THIS SHOULD BE ENCAPSULATED IN A SUBROUTINE
!1) get bands near Ef
!2) order irred kpoints 
!3) reconstruct full kgrid from irred kpoints,
!4) make tables from IBZ to FULLZONE
!4) setup weights for integration (gaussian or tetrahedron method) 
!5) calculate DOS at Ef
!====================================================================

 elphsmear_tmp = anaddb_dtset%elphsmear

 call get_fs_kpts(eigenGS,elph_ds,FSkptflag,elphsmear_tmp,hdr)

 if (elph_ds%ep_keepbands == 0) then !we are summing over bands
   elph_ds%ngkkband = 1
 else if (elph_ds%ep_keepbands == 1) then
!  use trivial integration weights since average over bands is done in normsq_gkk
!  and keep the band dependency btw elph_ds%minFS and elph_ds%maxFS) 
   elph_ds%ngkkband = elph_ds%nFSband
 else
   write(message,'(4a,i4)')ch10,&
&   ' elphon : BUG- ',ch10,     &
&   ' ep_keepbands must be 0 or 1 while it is : ',elph_ds%ep_keepbands
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if

 minFSband=elph_ds%minFSband
 maxFSband=elph_ds%maxFSband

 write(message,'(a,i4,2x,i4)')' elphon : minFSband, maxFSband = ',minFSband,maxFSband
 call wrtout(std_out,message,'COLL')

!Check consistency of arguments, and that FSintweight will be well dimensioned
 if (elph_ds%ep_keepbands == 1 .and. elph_ds%nFSband /= elph_ds%ngkkband) then
   write (*,*) 'elphon : Error: nFSband /= ngkkband'
   stop
 end if

 allocate(FSkptirred(3,elph_ds%nFSkptirred),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating FSkptirred'
 allocate(FSirredtoGS(elph_ds%nFSkptirred),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating FSirredtoGS'
 allocate(FSkptirrank(elph_ds%nFSkptirred),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating FSkptirrank'

 call order_fs_kpts(FSkptflag,FSkptirrank,FSkptirred,FSirredtoGS,&
& hdr,elph_ds%nFSkptirred)

!==========================================
!Set up full FSkpt grid from irred points
!==========================================

 allocate(tmpFSkpt(3,2*elph_ds%nFSkptirred*nsym),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating tmpFSkpt'
 allocate(tmpFSfulltoirred(3,2*elph_ds%nFSkptirred*nsym),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating tmpFSfulltoirred'
 allocate(tmpFSfulltofull(2,nsym,2*elph_ds%nFSkptirred*nsym),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating tmpFSfulltofull'
 allocate(FSirredwtk(elph_ds%nFSkptirred),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating FSirredwtk'
 allocate(FSirredtofull(elph_ds%nFSkptirred),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating FSirredtofull'

 call mkFSkgrid(FSirredwtk,FSirredtofull,tmpFSfulltoirred,&
& tmpFSfulltofull,tmpFSkpt,FSkptirred,&
& elph_ds%nFSkptirred,nFSkpt,nsym,symrec,timrev)

 elph_ds%nFSkpt = nFSkpt

!allocate FS arrays for kpoints only to size needed
 allocate(elph_ds%FSkpt(3,elph_ds%nFSkpt),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating elph_ds%FSkpt'
 allocate(FSkpt(3,elph_ds%nFSkpt),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating FSkpt'
 allocate(FSfulltofull(2,nsym,elph_ds%nFSkpt),FSfulltoirred(3,elph_ds%nFSkpt),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating FSfulltofull,FSfulltoirred'

!copy the kpoints and full --> irred FSkpt map
 do ikpt=1,elph_ds%nFSkpt
!  MG FIXME 
!  I dont like this part since we are duplicating FSkpt but I need elph_ds%FSkpt in my subroutines
!  In the next version the setup of the kgrid could be done in the new subroutines setkgrid.F90
!  and all these quantities should be saved in elph_ds%
   elph_ds%FSkpt(:,ikpt) = tmpFSkpt(:,ikpt)
!  ENDMG
   FSkpt(:,ikpt) = tmpFSkpt(:,ikpt)
   FSfulltoirred(:,ikpt) = tmpFSfulltoirred(:,ikpt)
   do isym=1,nsym
     FSfulltofull(:,isym,ikpt) = tmpFSfulltofull(:,isym,ikpt)
   end do
 end do
 
 deallocate(tmpFSkpt,tmpFSfulltoirred,tmpFSfulltofull)

!in spinor or spin polarized case, orbitals have occupation <= 1 instead of 2
 max_occ = one
 if (hdr%nspinor == 2) max_occ = half ! this accounts for the doubling of the num of bands, even though spin channels are not well defined
 if (elph_ds%nsppol > 1) max_occ = one
 write (*,*) ' max_occ factor  ', max_occ

 if (hdr%nspinor == 1 .and. hdr%nsppol == 1) then
   elph_ds%occ_factor = one
 else if (hdr%nspinor == 2) then
   elph_ds%occ_factor = two
 else if (hdr%nsppol == 2) then
   elph_ds%occ_factor = one
 end if

!
!NOTE: should start encapsulation of intweight calculation HERE
!
!===================================
!Set up integration weights for FS
!telphint = 0 or 1
!1 = gaussians
!0 = tetrahedron method
!===================================

 allocate(elph_ds%FSintweight(elph_ds%nFSband,elph_ds%nFSkpt,elph_ds%nsppol),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating elph_ds%FSintweight'
 allocate(elph_ds%n0(elph_ds%nsppol),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating elph_ds%n0'

 if (anaddb_dtset%telphint == 1) then

!  ==============================================================
!  Gaussian integration:
!  Each FSkpt contributes a gaussian of integrated weight 1 
!  for each band. The gaussian being centered at the Fermi level.
!  ===============================================================

!  gaussprefactor = sqrt(piinv)/anaddb_dtset%elphsmear/elph_ds%nFSkpt
!  took out factor 1/nkpt which intervenes only at integration time

!  MJV 18/5/2008 does gaussprefactor need to contain max_occ?

   gaussprefactor = max_occ*sqrt(piinv)/anaddb_dtset%elphsmear
   gaussfactor = one/anaddb_dtset%elphsmear
   elph_ds%FSintweight = zero
!  SPPOL loop on isppol as well to get 2 sets of weights
   do isppol=1,elph_ds%nsppol
     do iFSkpt=1,elph_ds%nFSkpt
       ikpt = FSirredtoGS(FSfulltoirred(1,iFSkpt))
       do ib1=1,elph_ds%nFSband
         xx = gaussfactor*(eigenGS(minFSband-1+ib1,ikpt,isppol) - elph_ds%fermie)
         elph_ds%FSintweight(ib1,iFSkpt,isppol) = exp(-xx*xx)*gaussprefactor
       end do
     end do
   end do
   

 else if (anaddb_dtset%telphint == 0) then

!  =========================
!  Tetrahedron integration
!  =========================

!  Get tetrahedra, ie indexes of the full kpoints at their summits
   mtetra = 6 * elph_ds%nFSkpt
   ntetra = mtetra
   
!  tetra_full(:,1,i) contains the irred kpt  number
!  tetra_full(:,2,i) contains the fullbz_kpt number
!  tetra_wrap(:,:,i) contains the a flag to wrap kpoints outside the IBZ (+-1)
!  to get the irreducible tetrahedra, the number of equivalent
!  tetrahedra is counted in tetra_mult and the inequivalent few
!  (ntetra < mtetra) are packed into the beginning of tetra_full
!  klatt here refer to the shortest kpt vectors, not qpt as below
!  convert kptrlatt to double and invert

   rlatt(:,:) = anaddb_dtset%kptrlatt(:,:)
   call matr3inv(rlatt,klatt)

   allocate (tetra_full(4,2,mtetra),tetra_wrap(3,4,mtetra),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating tetra_full,tetra_wrap'
   allocate(tetra_mult(mtetra),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating tetra_mult'

!  call get_tetra (FSfulltoirred(1,:),gprimd,klatt,FSkpt,mtetra,&
!  &       elph_ds%nFSkpt,ntetra,&
!  &       tetra_full,tetra_mult,tetra_wrap,vv)
!  
!  call get_tetra without using symmetries: trivial indkpt
!  
   call get_tetra(FSfulltofull(1,1,:),gprimd,klatt,FSkpt,mtetra,&
&   elph_ds%nFSkpt,ntetra,tetra_full,tetra_mult,tetra_wrap,vv)

   rcvol = abs (gprimd(1,1)*(gprimd(2,2)*gprimd(3,3)-gprimd(3,2)*gprimd(2,3)) &
&   -gprimd(2,1)*(gprimd(1,2)*gprimd(3,3)-gprimd(3,2)*gprimd(1,3)) &
&   +gprimd(3,1)*(gprimd(1,2)*gprimd(2,3)-gprimd(2,2)*gprimd(1,3)))

!  just do weights at FS
   nene = 100

!  fix small window around elph_ds%fermie for tetrahedron weight calculation
   deltaene = 2*anaddb_dtset%elphsmear/dble(nene-1)
   ifermi = int(nene/2)
   enemin = elph_ds%fermie - dble(ifermi-1)*deltaene
   enemax = enemin + dble(nene-1)*deltaene

   allocate(tmp_eigen(elph_ds%nFSkpt),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating elph_ds%nFSkpt'
   allocate(tweight(elph_ds%nFSkpt,nene),dtweightde(elph_ds%nFSkpt,nene),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating tweight,dtweightde'

   do iband = 1,elph_ds%nFSband
!    for each spin pol
     do isppol=1,elph_ds%nsppol
!      For this band get its contribution
       tmp_eigen(:) = zero
       do iFSkpt=1,elph_ds%nFSkpt
         ikpt = FSirredtoGS(FSfulltoirred(1,iFSkpt))
         tmp_eigen(iFSkpt) = eigenGS(minFSband+iband-1,ikpt,isppol)
       end do
!      calculate general integration weights at each irred kpoint as in Blochl et al PRB 49 16223
       call get_tetra_weight(tmp_eigen,enemin,enemax,max_occ,mtetra,nene,elph_ds%nFSkpt,&
&       ntetra,rcvol,tetra_full,tetra_mult,tweight,dtweightde,vv)
       elph_ds%FSintweight(iband,:,isppol) = dtweightde(:,ifermi)*elph_ds%nFSkpt
     end do

   end do
   deallocate(tmp_eigen,tweight,dtweightde)
   deallocate(tetra_full,tetra_wrap,tetra_mult)


 else if (anaddb_dtset%telphint == 2) then ! range of bands occupied

!  SPPOL eventually be able to specify bands for up and down separately
   elph_ds%FSintweight = zero
   do iFSkpt=1,elph_ds%nFSkpt
     ikpt = FSirredtoGS(FSfulltoirred(1,iFSkpt))
     do ib1=anaddb_dtset%ep_b_min,anaddb_dtset%ep_b_max
!      for the moment both spin channels same
       elph_ds%FSintweight(ib1,iFSkpt,:) = max_occ
     end do
   end do
   
   write (*,*) ' elphon : DOS is calculated from states in bands ',&
&   anaddb_dtset%ep_b_min,' to ',anaddb_dtset%ep_b_max
   
 else 
   write (message,'(4a)')ch10,&
&   ' elphon : ERROR-',ch10,&
&   'telphint should be 0 1 or 2 '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if !of if telphint

!SPPOL sum over spin channels to get total DOS
!channels decoupled => use separate values for DOS_up(Ef) resp down
 do isppol=1,elph_ds%nsppol
   elph_ds%n0(isppol) = sum(elph_ds%FSintweight(:,:,isppol))/elph_ds%nFSkpt
 end do

 if (elph_ds%nsppol == 1) then
   write (*,*) ' elphon : the estimated DOS(E_Fermi) = ', elph_ds%n0(1), ' states/Ha/spin '
   write (*,*) ' elphon : the total FS weight and # of kpoints = ',sum(elph_ds%FSintweight),elph_ds%nFSkpt
 else if (elph_ds%nsppol == 2) then
   write (*,*) ' elphon : the spin up   DOS(E_Fermi) = ', elph_ds%n0(1), ' states/Ha/spin '
   write (*,*) ' elphon : the spin down DOS(E_Fermi) = ', elph_ds%n0(2), ' states/Ha/spin '
   write (*,*) ' elphon : total DOS(E_Fermi) = ', elph_ds%n0(1)+elph_ds%n0(2), ' states/Ha '
   write (*,*) ' elphon : the spin up   FS weight and # of kpoints = ',&
&   sum(elph_ds%FSintweight(:,:,1)),elph_ds%nFSkpt
   write (*,*) ' elphon : the spin down FS weight and # of kpoints = ',&
&   sum(elph_ds%FSintweight(:,:,2)),elph_ds%nFSkpt
 else
   write (*,*) 'bad value for nsppol ', elph_ds%nsppol
   stop
 end if

 allocate(elph_ds%gkk_intweight(elph_ds%ngkkband,elph_ds%nFSkpt,elph_ds%nsppol),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating elph_ds%gkk_intweight'

 if (elph_ds%ep_keepbands == 0) then
!  use trivial integration weights  for single band,
!  since average over bands is done in normsq_gkk
   elph_ds%gkk_intweight(1,:,:) = one

 else if (elph_ds%ep_keepbands == 1) then
!  use elph_ds%FSintweight since average over bands is not done in normsq_gkk
   elph_ds%gkk_intweight(:,:,:) = elph_ds%FSintweight(:,:,:)
 else
   write(message,'(4a,i4)')ch10,' elphon : BUG- ',ch10,            &
&   ' ep_keepbands must be 0 or 1 while it is : ',elph_ds%ep_keepbands
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if

!
!NOTE: should end encapsulation of intweight calculation HERE
!


!Output of the Fermi Surface
 if (anaddb_dtset%prtfsurf == 1) then
   fname=trim(elph_ds%elph_base_name) // '_BXSF'

!  FIXME
!  shiftk is not defined neither in the anaddb nor in the hdr data type
!  an incorrect FS will be produced in case of a shifted k-grid used during the GS calculation
!  check if we are using a unshifthed kgrid, obviously doesnt work in case
!  of multiple shifts containg a zero translation but in this case prtbxsf should work
   shiftk=one
   do ii=1,hdr%nkpt
     if (all(hdr%kptns(:,ii) == zero)) shiftk=zero
   end do

!  the input argument in printbxsf has shape (nband,nkpt,nspin), to be on the safe side we use:
   allocate (dummy_eig(nband,hdr%nkpt,elph_ds%nsppol),stat=istat)
   if (istat /= 0) stop 'elphon: error in allocating dummy_eig'
   dummy_eig(:,:,:)=eigenGS(:,:,:)

   use_afm=(Hdr%nsppol==1.and.Hdr%nspden==2)
!  MG FIXME warning time reversal is always assumed to be present. 
!  the header should report this information.

   call printbxsf(dummy_eig,zero,elph_ds%fermie,gprimd,&
&   anaddb_dtset%kptrlatt,nband,hdr%nkpt,hdr%kptns,&
&   nsym,use_afm,symrec,Hdr%symafm,timrev,elph_ds%nsppol,shiftk,1,fname)

   deallocate (dummy_eig)
 end if !anaddb_dtset%prtfsurf

!=========================================================    
!Get equivalence between a FSkpt pair and a qpt in qpt_full
!only works if the qpt grid is complete (identical to
!the kpt one, with a basic shift of (0,0,0)
!=========================================================    

 allocate(FSfullpqtofull(elph_ds%nFSkpt,elph_ds%nqpt_full),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating FSfullpqtofull'
 allocate(qpttoqpt(2,nsym,elph_ds%nqpt_full),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating qpttoqpt'

 write (message,'(2a)')ch10,' elphon : calling mkqptequiv to set up the FS qpoint set'
 call wrtout(std_out,message,'COLL')

 call mkqptequiv (FSfullpqtofull,FSkpt,elph_ds%nFSkpt,&
& elph_ds%nqpt_full,nsym,qpttoqpt,elph_ds%qpt_full,symrec)


!==========================================
!Set up dataset for phonon interpolations
!==========================================

 call setup_phon_ds(phon_ds,anaddb_dtset%dipdip,mpert,nsym,natom,ntypat,nrpt,&
& ucvol,indsym,symrel,typat,acell,amu,atmfrc,dielt,dyewq0,gprim,gmet,&
& xred,zeff,rcan,rmet,rprim,rprimd,rpt,trans,wghatm)

!transfer ifltransport flag to structure
 elph_tr_ds%ifltransport=anaddb_dtset%ifltransport
!transfer name of files file for ddk
 elph_tr_ds%ddkfilename=ddkfilename

!
!order fs qpts as well!
!
!call order_fs_kpts(FSkptflag,FSkptirrank,FSkptirred,FSirredtoGS,&
!& hdr,elph_ds%nFSkptirred)


!reduce qpt_full to correct zone
 do iqpt=1,elph_ds%nqpt_full
   call wrap2_pmhalf(elph_ds%qpt_full(1,iqpt),kpt(1),res)
   call wrap2_pmhalf(elph_ds%qpt_full(2,iqpt),kpt(2),res)
   call wrap2_pmhalf(elph_ds%qpt_full(3,iqpt),kpt(3),res)
   elph_ds%qpt_full(:,iqpt)=kpt
 end do


!===================================================
!Allocate all important arrays for FS integrations
!===================================================

!SPPOL allocate with additional dimension for sppol?
 allocate(gkk_flag(elph_ds%nbranch,elph_ds%nbranch,elph_ds%nFSkpt,&
& elph_ds%nsppol,elph_ds%nqpt_full),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating gkk_flag'

!Record sizes for matrices on disk: complex and real versions (for real and recip space resp!)
 onegkksize = 2*elph_ds%nbranch*elph_ds%nbranch*&
& elph_ds%ngkkband*elph_ds%ngkkband*&
& elph_ds%nsppol*kind(realdp_ex)
!& elph_ds%nFSkpt*elph_ds%nsppol*kind(realdp_ex)
 elph_tr_ds%onegkksize=onegkksize

!MJV 18/5/2008 : looks like onegkk2size is identical to onegkksize - is it
!used?
 onegkk2size =  elph_ds%nbranch*elph_ds%nbranch*&
& elph_ds%ngkkband*elph_ds%ngkkband*&
& elph_ds%nFSkpt*elph_ds%nsppol*kind(realdp_ex)

 write (message,'(a,3(a,i12,a))')ch10,               &
& ' elphon : kind(dp real)   = ',kind(realdp_ex),ch10,&
& '               onegkksize = ',onegkksize,ch10,     &
& '              onegkk2size = ',onegkk2size,ch10
 call wrtout(std_out,message,'COLL')

 allocate (qptirred(3,n1wf),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating qptirred'

 write (message,'(5a)')                         &
& ' elphon : preliminary setup completed ',ch10,&
& '          calling get_all_gkq to read in',&
& ' all the e-ph matrix elements',ch10
 call wrtout(std_out,message,'COLL')

!flag to do scalar product in gkq before interpolation:
!should also used in interpolate_gkk and mkph_linwid
 elph_ds%ep_scalprod=anaddb_dtset%ep_scalprod
 if (elph_ds%ep_scalprod==0) then
   write (*,*) ' elphon: will NOT perform scalar product with phonon'
   write (*,*) '  displacement vectors in read_gkk. ep_scalprod==0'
 else if (elph_ds%ep_scalprod==1) then
   write (*,*) ' elphon: will perform scalar product with phonon'
   write (*,*) '  displacement vectors in read_gkk. ep_scalprod==1'
 else
   write (*,*) ' elphon: illegal value for ep_scalprod'
   stop
 end if

 call get_all_gkq (elph_ds,FSfullpqtofull,FSfulltofull,&
& FSkpt,elph_ds%FSintweight,gkk_flag,gprimd,indsym,natom,&
& nband,nsym,n1wf,onegkksize,phon_ds,&
& qpttoqpt,rprimd,symrec,symrel,tnons,&
& anaddb_dtset%ep_prt_yambo,&
& unitgkk)


 if (elph_tr_ds%ifltransport==1 )then
   call get_gkk_qpt_tr(elph_ds,mpi_enreg,nband,hdr,FSfulltoirred, FSirredtoGS,FSfullpqtofull,elph_tr_ds)
 end if

!============================================================================
!Evaluate lambda and omega_log using the weighted sum over the irred q-points
!found in the GKK file. All the data we need are stored in elph_ds%qgrid_data
!============================================================================
 write(message,'(a)')' elphon: calling outelph '
 call wrtout(std_out,message,'COLL')
 fname=trim(elph_ds%elph_base_name) // '_QPTS'
 call outelph(elph_ds,anaddb_dtset%enunit,fname)

!========================================================
!Get FS averaged gamma matrices and Fourier transform to real space 
!========================================================
 
!NOTE: gprim (not gprimd) is used for all FT interpolations,
!to be consistent with the dimensions of the rpt, which come from anaddb.
 call integrate_gamma(elph_ds,FSfullpqtofull,nrpt)

 if (elph_ds%symgkq ==1) then
!  complete the gamma_qpt here instead of the gkk previously
   call complete_gamma(elph_ds,elph_ds%gamma_qpt, &
&   gprimd,indsym,natom,nsym,qpttoqpt,rprimd, &
&   symrec,symrel)
 end if

!Now FT to real space too
 qtor = 1 ! q --> r
 do isppol=1,elph_ds%nsppol
   call ftgam(wghatm,elph_ds%gamma_qpt(:,:,isppol,:),elph_ds%gamma_rpt(:,:,isppol,:),gprim,natom,&
&   elph_ds%nqpt_full,nrpt,qtor,rpt,elph_ds%qpt_full)
 end do


!============================================
!Get q path for Fourier interpolation where 
!ph linewidths are going to be interpolated
!TODO MJV 5/8/2007 could be made into a separate subroutine
!
!MG Sat 02/05/2009, we can use make_path in m_bz_mesh
!
!from HERE
!
!============================================

 nsegment = anaddb_dtset%nqpath-1
 allocate (npoint(nsegment),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating npoint'

!get reference squared distance |q2-q1|^2 and calculate npoint for each segment
!MG In the previous version 5.1.3  there was a bug, since we were evaluating npoint as
!(tmpdist/refdist)^2 instead of the correct ratio (tmpdist/refdist)

 tmpkpt(:) = anaddb_dtset%qpath(:,2)-anaddb_dtset%qpath(:,1)
 refdist =  gmet(1,1)*tmpkpt(1)*tmpkpt(1) &
& +gmet(2,2)*tmpkpt(2)*tmpkpt(2) &
& +gmet(3,3)*tmpkpt(3)*tmpkpt(3) &
& +two*gmet(1,2)*tmpkpt(1)*tmpkpt(2) &
& +two*gmet(1,3)*tmpkpt(1)*tmpkpt(3) &
& +two*gmet(2,3)*tmpkpt(2)*tmpkpt(3)

 if (refdist <= tol10 ) then
   write (message,'(8a)')ch10,&
&   ' elphon : ERROR-',ch10,&
&   ' The first and second q point of the path are equal ',ch10,&
&   ' Impossible to calculate the reference distance. ',ch10,   &
&   ' Modify the q path in your input file '
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
 end if

 factor = 100.0_dp
 do ii=1,anaddb_dtset%nqpath-1
   tmpkpt(:) = anaddb_dtset%qpath(:,ii+1)-anaddb_dtset%qpath(:,ii)
   tmpdist =   gmet(1,1)*tmpkpt(1)*tmpkpt(1) &
&   +gmet(2,2)*tmpkpt(2)*tmpkpt(2) &
&   +gmet(3,3)*tmpkpt(3)*tmpkpt(3) &
&   +two*gmet(1,2)*tmpkpt(1)*tmpkpt(2) &
&   +two*gmet(1,3)*tmpkpt(1)*tmpkpt(3) &
&   +two*gmet(2,3)*tmpkpt(2)*tmpkpt(3)
   npoint(ii) = nint(factor*(sqrt(tmpdist/refdist)))
 end do

 write (*,*) ' elphon : npoint = ', npoint

!
!to HERE: end sub for anaddb_dtset%qpath creation
!

!==========================================================
!calculate transport matrix elements, integrated over FS
!==========================================================
 if (elph_tr_ds%ifltransport==1 )then
   call integrate_gamma_tr(elph_ds,FSfullpqtofull,nrpt,elph_tr_ds)

   call complete_gamma(elph_ds,elph_tr_ds%gamma_qpt_trout, &
&   gprimd,indsym,natom,nsym,qpttoqpt,rprimd, &
&   symrec,symrel)

   call complete_gamma(elph_ds,elph_tr_ds%gamma_qpt_trin, &
&   gprimd,indsym,natom,nsym,qpttoqpt,rprimd, &
&   symrec,symrel)

!  Now FT to real space too
   qtor = 1 ! q --> r
   do isppol=1,elph_ds%nsppol
     call ftgam(wghatm,elph_tr_ds%gamma_qpt_trout(:,:,isppol,:),&
&     elph_tr_ds%gamma_rpt_trout(:,:,isppol,:),gprim,natom,&
&     elph_ds%nqpt_full,nrpt,qtor,rpt,elph_ds%qpt_full)

     call ftgam(wghatm,elph_tr_ds%gamma_qpt_trin(:,:,isppol,:),&
&     elph_tr_ds%gamma_rpt_trin(:,:,isppol,:),gprim,natom,&
&     elph_ds%nqpt_full,nrpt,qtor,rpt,elph_ds%qpt_full)
   end do

 end if

!==============================================================
!Calculate phonon linewidths, interpolating on chosen qpoints
!==============================================================

 call mkph_linwid(elph_ds,&
& gprim,gprimd,natom,npoint,nrpt,&
& nsegment,phon_ds,anaddb_dtset%qpath,rpt,wghatm)


!==============================================================
!the nesting factor calculation
!==============================================================
 if (anaddb_dtset%prtnest==1 .or. anaddb_dtset%prtnest==2) then
   write(message,'(a)')&
&   ' elphon : calling mknesting to interpolate the nesting factor'
   call wrtout(std_out,message,'COLL')

   call mknesting(elph_ds%nFSkpt,FSkpt,anaddb_dtset%kptrlatt,elph_ds%nFSband,&
&   elph_ds%FSintweight,nsegment,npoint,anaddb_dtset%qpath,elph_ds%elph_base_name,&
&   gprimd,anaddb_dtset%prtnest)
 end if
 
!======================================================
!Calculate alpha^2 F integrating over fine FSkpt grid
!======================================================

 allocate(a2f_1d(elph_ds%na2f),dos_phon(elph_ds%na2f),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating a2f_1d,dos_phon'
 
 write(message,'(a)')' elphon : calling mka2f '
 call wrtout(std_out,message,'COLL')

 call mka2f(a2f_1d,dos_phon,elph_ds,&
& FSkpt,gprim,gprimd,anaddb_dtset%mustar,&
& elph_ds%n0,natom,nrpt,phon_ds,rpt,wghatm)

!calculate transport spectral function and coefficients
 if (elph_tr_ds%ifltransport==1 )then
   call mka2f_tr(elph_ds,FSkpt,gprim,gprimd,elph_ds%n0,ucvol,natom,nrpt,&
&   anaddb_dtset%ntemper,anaddb_dtset%tempermin,anaddb_dtset%temperinc,&
&   phon_ds,rpt,wghatm,elph_tr_ds)


!  if transport gkk are kept in memory, deallocate now.
   if (elph_ds%gkqwrite == 0) then
     deallocate (elph_tr_ds%eta_trin)
     deallocate (elph_tr_ds%eta_trout)
   end if
 end if

!MG: evaluate a2F only using the input Q-grid (without using interpolated matrices)
!SCOPE: test the validity of the Fourier interpolation
!
 write(message,'(a)')' elphon : calling mka2fQgrid'
 call wrtout(std_out,message,'COLL')
 fname=trim(elph_ds%elph_base_name) // '_A2F_QGRID'
 call mka2fQgrid(elph_ds,fname)
!ENDMG

!=============================================
!Eliashberg equation in 1-D (isotropic case)
!=============================================

 write (message,'(2a)')ch10,&
& ' Solving the 1-D Eliashberg equation (isotropic case)'
 call wrtout(std_out,message,'COLL')

 call eliashberg_1d(a2f_1d,elph_ds,anaddb_dtset%mustar)


 deallocate (a2f_1d,dos_phon)

!MJV: 20070805 should exit here. None of the rest is tested or used yet to my knowledge

!========================================================================
!Now gkk contains the matrix elements of dH(1)/dxi i=1,2,3
!for kpoints on the FS but qpoints only in the given grid {Q}.
!
!1.) Need to complete the gkk elements for q and k\prime=k+q not 
!in the set of {k+Q} by Fourier interpolation on the Q.
!
!2.) Need to complete the dynamical matrices and phonon freqs for
!all q between points on the FS.
!
!3.) With the eigenvectors e_ph of the dyn mats, do the scalar product
!e_ph . gkk, which implies the gkk are turned to the eigenbasis of
!the phonons. Before the (non eigen-) modes are ordered
!atom1 xred1 atom1 xred2 atom1 xred3
!atom2 xred1 atom2 xred2 atom2 xred3 ...
!=======================================================================

 make_gkk2=.false.
 
 if (.not. make_gkk2) then
   write(message,'(2a)')ch10,&
&   ' elphon : skipping full g(k,k") interpolation '
   call wrtout(std_out,message,'COLL')
 else

!  ==========================================================
!  FT of recip space gkk matrices to real space (gkk_rpt)
!  NOTE: could be made into FFT, couldnt it? If shifts are
!  used with a homogeneous grid
!  ==========================================================
   write (message,'(2a,i5)')ch10,&
&   ' elphon : Fourier transform (q --> r) of the gkk matrices using nrpt = ',nrpt
   call wrtout(std_out,message,'COLL')

   call get_all_gkr(elph_ds,gprim,natom,nrpt,onegkksize,rpt,elph_ds%qpt_full,wghatm)

!  =========================================================
!  complete gkk2 for all qpts between points
!  on full kpt grid (interpolation from real space values)
!  =========================================================

   write(message,'(2a)')ch10,&
&   ' elphon : Calling get_all_gkk2 to calculate gkk2 for q points over the full k grid'
   call wrtout(std_out,message,'COLL')
   
   call get_all_gkk2(elph_ds,FSkptirred,FSkpt,   &
&   natom,nrpt, &
&   phon_ds,rcan,&
&   wghatm)
 end if


 allocate(zz(elph_ds%na2f,elph_ds%nFSkpt),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating zz'
 allocate(delta(elph_ds%na2f,elph_ds%nFSkpt),stat=istat)
 if (istat /= 0) stop 'elphon: error in allocating delta'
 
!=====================================================
!Here should be the anisotropic Eliashberg equations.
!=====================================================

!initialize delta function

!initialize T_c

!initialize delta

!iterate for calculation of T_c
 do ieliash=1,neliash

!  ===========================
!  calculate lambda function
!  ===========================

!  ========================================
!  integrate lambda over FS -> Z function
!  ========================================
   
!  ========================================
!  integrate delta*Z over FS -> new delta
!  ========================================
   
!  update T_c

 end do

!end iterate ieliash

!output T_c and related quantities


!clean and deallocate junk

 deallocate (elph_ds%wtq)
 deallocate(elph_ds%qpt_full)

 call clean_phon_ds(phon_ds)

 call hdr_clean(hdr)

 end subroutine elphon
!!***

